Struct DagAnalysis
pub struct DagAnalysis<N, S = FixedHasher>where
N: GraphNodeId,
S: BuildHasher,{ /* private fields */ }Expand description
Stores the results of a call to Dag::analyze.
Implementations
impl<N, S> DagAnalysis<N, S>where
N: GraphNodeId,
S: BuildHasher,
impl<N, S> DagAnalysis<N, S>where
N: GraphNodeId,
S: BuildHasher,
pub fn new(
graph: &Graph<true, N, S>,
topological_order: &[N],
) -> DagAnalysis<N, S>where
S: Default,
pub fn new(
graph: &Graph<true, N, S>,
topological_order: &[N],
) -> DagAnalysis<N, S>where
S: Default,
Processes a DAG and computes its:
- transitive reduction (along with the set of removed edges)
- transitive closure
- reachability matrix (as a bitset)
- pairs of nodes connected by a path
- pairs of nodes not connected by a path
The algorithm implemented comes from “On the calculation of transitive reduction-closure of orders” by Habib, Morvan and Rampon.
pub fn reachable(&self) -> &FixedBitSet
pub fn reachable(&self) -> &FixedBitSet
Returns the reachability matrix.
pub fn connected(&self) -> &HashSet<(N, N), S>
pub fn connected(&self) -> &HashSet<(N, N), S>
Returns the set of node pairs that are connected by a path.
pub fn disconnected(&self) -> &[(N, N)]
pub fn disconnected(&self) -> &[(N, N)]
Returns the list of node pairs that are not connected by a path.
pub fn transitive_edges(&self) -> &[(N, N)]
pub fn transitive_edges(&self) -> &[(N, N)]
Returns the list of redundant edges because a longer path exists.
pub fn transitive_reduction(&self) -> &Graph<true, N, S>
pub fn transitive_reduction(&self) -> &Graph<true, N, S>
Returns the transitive reduction of the graph.
pub fn transitive_closure(&self) -> &Graph<true, N, S>
pub fn transitive_closure(&self) -> &Graph<true, N, S>
Returns the transitive closure of the graph.
pub fn check_for_redundant_edges(&self) -> Result<(), DagRedundancyError<N>>where
S: Clone,
pub fn check_for_redundant_edges(&self) -> Result<(), DagRedundancyError<N>>where
S: Clone,
Checks if the graph has any redundant (transitive) edges.
Errors
If there are redundant edges, returns a DagRedundancyError
containing the list of redundant edges.
pub fn check_for_cross_dependencies(
&self,
other: &DagAnalysis<N, S>,
) -> Result<(), DagCrossDependencyError<N>>
pub fn check_for_cross_dependencies( &self, other: &DagAnalysis<N, S>, ) -> Result<(), DagCrossDependencyError<N>>
Checks if there are any pairs of nodes that have a path in both this graph and another graph.
Errors
Returns DagCrossDependencyError if any node pair is connected in
both graphs.
pub fn check_for_overlapping_groups<K, V>(
&self,
groups: &DagGroups<K, V>,
) -> Result<(), DagOverlappingGroupError<K>>
pub fn check_for_overlapping_groups<K, V>( &self, groups: &DagGroups<K, V>, ) -> Result<(), DagOverlappingGroupError<K>>
Checks if any connected node pairs that are both keys have overlapping groups.
Errors
If there are overlapping groups, returns a DagOverlappingGroupError
containing the first pair of keys that have overlapping groups.